@@ -6,7 +6,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-0.55
+0.56
 
 =head1 SYNOPSIS
 
@@ -142,7 +142,7 @@ use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG} || $ENV{SWAGGER2_DEBUG} || 0;
 use constant WARN_ON_MISSING_FORMAT => $ENV{JSON_VALIDATOR_WARN_ON_MISSING_FORMAT}
   || $ENV{SWAGGER2_WARN_ON_MISSING_FORMAT} ? 1 : 0;
 
-our $VERSION = '0.55';
+our $VERSION = '0.56';
 our @EXPORT_OK = qw( validate_json );
 
 my $HTTP_SCHEME_RE = qr{^https?:};
@@ -180,15 +180,6 @@ Path to where downloaded spec files should be cached. Defaults to
 C<JSON_VALIDATOR_CACHE_DIR> or the bundled spec files that are shipped
 with this distribution.
 
-=head2 coerce
-
-  $self = $self->coerce(1);
-  $bool = $self->coerce;
-
-Set this to true if you want to coerce numbers into string and the other way around.
-
-This is EXPERIMENTAL and could be removed without notice!
-
 =head2 formats
 
   $hash_ref = $self->formats;
@@ -247,8 +238,6 @@ has cache_dir => sub {
   $ENV{JSON_VALIDATOR_CACHE_DIR} || File::Spec->catdir(File::Basename::dirname(__FILE__), qw( JSON Validator ));
 };
 
-has coerce => $ENV{JSON_VALIDATOR_COERCE_VALUES} || $ENV{SWAGGER_COERCE_VALUES} || 0;    # EXPERIMENTAL!
-
 has formats => sub { shift->_build_formats };
 
 has ua => sub {
@@ -261,6 +250,46 @@ has ua => sub {
 
 =head1 METHODS
 
+=head2 coerce
+
+  $self = $self->coerce(booleans => 1, numbers => 1, strings => 1);
+  $self = $self->coerce(1) # enable all
+  $hash = $self->coerce;
+
+Set the given type to coerce. Before enabling coercion this module is very
+strict when it comes to validating types. Example: The string C<"1"> is not
+the same as the number C<1>, unless you have coercion enabled.
+
+WARNING! Enabling coercion might hide bugs in your api, which would have been
+detected if you were strict. For example JavaScript is very picky on a number
+being an actual number. This module tries it best to convert the data on the
+fly into the proper value, but this means that you unit tests might be ok,
+but the client side libraries (that care about types) might break.
+
+Loading a YAML document will enable "booleans" automatically. This feature is
+experimental, but was added since YAML has no real concept of booleans, such
+as L<Mojo::JSON> or other JSON parsers.
+
+The coercion rules are EXPERIMENTAL and will be tighten/loosen if
+bugs are reported. See L<https://github.com/jhthorsen/json-validator/issues/8>
+for more details.
+
+=cut
+
+sub coerce {
+  my ($self, @args) = @_;
+
+  return $self->{coerce} ||= {} unless @args;
+
+  @args = (booleans => 1, numbers => 1, strings => 1) if $args[0] eq '1';    # back compat
+  while (@args) {
+    my $k = shift @args;
+    $self->{coerce}{$k} = shift @args;
+  }
+
+  return $self;
+}
+
 =head2 schema
 
   $self = $self->schema(\%schema);
@@ -413,7 +442,9 @@ sub _load_schema_from_data {
 }
 
 sub _load_schema_from_text {
-  $_[1] =~ /^\s*\{/s ? Mojo::JSON::decode_json($_[1]) : _load_yaml($_[1]);
+  return Mojo::JSON::decode_json($_[1]) if $_[1] =~ /^\s*\{/s;
+  $_[0]->{coerce}{booleans} = 1;    # internal coercion
+  _load_yaml($_[1]);
 }
 
 sub _load_schema_from_url {
@@ -640,6 +671,7 @@ sub _validate_type_enum {
   my $m    = S $data;
 
   for my $i (@$enum) {
+    return if !$self->_validate_type_boolean($data, $path) and _is_true($data) == _is_true($i);
     return if $m eq S $i;
   }
 
@@ -725,7 +757,16 @@ sub _validate_type_array {
 sub _validate_type_boolean {
   my ($self, $value, $path, $schema) = @_;
 
-  return if defined $value and Scalar::Util::blessed($value) and ("$value" eq "1" or "$value" eq "0");
+  if (defined $value) {
+    if (Scalar::Util::blessed($value) and ("$value" eq "1" or !$value)) {
+      return;
+    }
+    if ($self->{coerce}{booleans} and (B::svref_2object(\$value)->FLAGS & B::SVp_NOK or $value =~ /^(true|false)$/)) {
+      $_[1] = $value ? Mojo::JSON->true : Mojo::JSON->false;
+      return;
+    }
+  }
+
   return E $path, _expected(boolean => $value);
 }
 
@@ -755,7 +796,7 @@ sub _validate_type_number {
     return E $path, _expected($expected => $value);
   }
   unless (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $value eq $value and $value * 0 == 0) {
-    return E $path, "Expected $expected - got string." if !$self->coerce or $value =~ /\D/;
+    return E $path, "Expected $expected - got string." if !$self->{coerce}{numbers} or $value =~ /\D/;
     $_[1] = 0 + $value;    # coerce input value
   }
 
@@ -841,7 +882,7 @@ sub _validate_type_string {
     return E $path, _expected(string => $value);
   }
   if (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $value eq $value and $value * 0 == 0) {
-    return E $path, "Expected string - got number." unless $self->coerce;
+    return E $path, "Expected string - got number." unless $self->{coerce}{strings};
     $_[1] = "$value";    # coerce input value
   }
   if ($schema->{format}) {
@@ -941,8 +982,9 @@ sub _is_ipv4 {
 sub _is_ipv6 { warn "Data::Validate::IP is not installed"; return; }
 
 sub _is_true {
-  return $_[0] if ref $_[0] and !Scalar::Util::blessed($_[0]);
-  return 0 if !$_[0] or $_[0] =~ /^(n|false|off)/i;
+  local $_ = $_[0];
+  return 0 + $_ if ref $_ and !Scalar::Util::blessed($_);
+  return 0 if !$_ or /^(n|false|off)/i;
   return 1;
 }
 
@@ -951,9 +993,11 @@ sub _is_uri { $_[0] =~ qr!^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*
 # Please report if you need to manually monkey patch this function
 # https://github.com/jhthorsen/json-validator/issues
 sub _load_yaml {
-  my @YAML_MODULES = qw( YAML::XS YAML::Syck YAML::Tiny YAML );        # subject to change
-  my $YAML_MODULE = (grep { eval "require $_;1" } @YAML_MODULES)[0];
-  die "Need to install a YAML module: @YAML_MODULES" unless $YAML_MODULE;
+  require List::Util;
+  my @YAML_MODULES = qw( YAML::XS YAML::Syck YAML::Tiny YAML );                     # subject to change
+  my $YAML_MODULE = (List::Util::first { eval "require $_;1" } @YAML_MODULES)[0];
+  die "Need to install one of these YAML modules: @YAML_MODULES (YAML::XS is recommended)" unless $YAML_MODULE;
+  warn "[JSON::Validator] Using $YAML_MODULE to parse YAML\n" if DEBUG;
   Mojo::Util::monkey_patch(__PACKAGE__, _load_yaml => eval "\\\&$YAML_MODULE\::Load");
   _load_yaml(@_);
 }
